# -*- coding: UTF-8 -*-
"""HistoricalCollector.

Class responsible to collect data from Twitter database or real-time post.

 (c) J.Wirlino and A.Adativa
 Date July 15th 2016
 https://github.com/JosielWirlino/SocialCrawler

"""

from termcolor import colored
import tweepy
import json
import sys
import time


class Collector:
    """Class to connect with TWitter to get data.

    Note:
        More information about Twitter's crentials
        see https://dev.twitter.com/overview/documentation

    Args:
        api_key (str) : api key Twitter's credential.
        api_secret (str) : api secret Twitter's credential.
        access_token (str) : Twitter's crential acess token.
        access_secret (str) : Twitter's crential acess secret.

    Attributes:
        api_key (str) : api key Twitter's credential.
        api_secret (str) : api secret Twitter's credential.
        access_token (str) : Twitter's crential acess token.
        access_secret (str) : Twitter's crential acess secret.

    """

    #   Global variable to count the tweets received

    stored_data_count = 0
    stream_data_count = 0

    def __init__(self, api_key=None, api_secret=None, access_token=None, access_secret=None):
        """When instatiated must be pass all parameters.
        
        If some parameter is None will be throw an error message and
        will be called sys.exit()

        Args:
            api_key (str) : api key Twitter's credential.
            api_secret (str) : api secret Twitter's credential.
            access_token (str) : Twitter's crential acess token.
            access_secret (str) : Twitter's crential acess secret.

        Attributes:
            api_key (str) : api key Twitter's credential.
            api_secret (str) : api secret Twitter's credential.
            access_token (str) : Twitter's crential acess token.
            access_secret (str) : Twitter's crential acess secret.
        
        """
        if(api_key is None or api_secret is None or access_token is None or access_secret is None):
            print(colored('Error: Any parameter can not be None', 'red'))
            sys.exit()

        # setting all parameters
        self.api_key = api_key
        self.api_secret = api_secret
        self.access_token = access_token
        self.access_secret = access_secret

        # requesting authentication

        self.auth = tweepy.OAuthHandler(self.api_key, self.api_secret)
        self.auth.set_access_token(self.access_token, self.access_secret)
        self.twitter = tweepy.API(self.auth)


    def getStoredData(self, query=None, since=None, until=None,city=None, geocode=None, save_path=None,result_limits='infinite', mode='items'):
        """Method to get data stored in Twitter dataset.

        There are two ways to get data: stored and stream
        This method (getStoredData) use Cursor to get all data
        
        #see http://tweepy.readthedocs.io/en/v3.5.0/cursor_tutorial.html
        
        Args:
            query (str) : string content your consult
            since (str) : begin date to start collect data
            until (str) : end date of the tweets to be collected
            city  (str) : specific city data
            geocode (str) : specific region of the city
            mode  (str) : define if you want items or page

        """
        if(query is None or since is None or until is None or save_path is None):
            print(colored('Error: query, since, until, and save_path parameters can not be None', 'red'))
            sys.exit()

        try:
            if(city is not None):
                log_file = open(save_path + 'log_from_' + city.replace(" ", "") +
                                '__' + since + '__' + until + '.tsv', 'a', encoding='utf8')
            else:
                log_file = open(save_path + 'log_' + since +
                                '__' + until + '.tsv', 'a', encoding='utf8')
        except IOError:
            print(colored('Cound not open file!'))
            sys.exit()
        # check if the query is about a specific city
        if(city is None):
            if(mode == 'items'):
                results = tweepy.Cursor(self.twitter.search,
                                        q=query,
                                        since=since,
                                        until=until).items()
            else:
                results = tweepy.Cursor(self.twitter.search,
                                        q=query,
                                        since=since,
                                        until=until).pages()
        else:
            if(mode == 'items'):
                results = tweepy.Cursor(self.twitter.search,
                                        q=query,
                                        since=since,
                                        until=until,
                                        geocode=geocode).items()
            else:
                results = tweepy.Cursor(self.twitter.search,
                                        q=query,
                                        since=since,
                                        until=until,
                                        geocode=geocode).pages()
        #writing column headers
        log_file.write("data_count"+"\t"+"twitter_user_id"+"\t"+"city"+"\t"+"tweet_created_at"+"\t"+"tweet.text"+"\n")
        while True:
            try:
                tweet = results.next()
                self.stored_data_count += 1

                print(colored(self.stored_data_count, 'red'), colored(
                    city, 'green'), colored(tweet.created_at, 'red'), tweet.text)
                # print ('\n')

                log_file.write(str(self.stored_data_count) + '\t' + str(tweet.user.id) + '\t' + str(
                    city) + '\t' + str(tweet.created_at) + '\t' + str(tweet.text.encode('utf-8')) + '\n')

            # wait 16 minute to search again
            # https://dev.twitter.com/rest/public/rate-limiting
            except tweepy.TweepError:
                print('\n')
                print(
                    colored('Twitter API rate limit. Wait 16 minutes to request again.\n ', 'red'))
                time.sleep(960)
                continue
            except StopIteration:
                break
        # when arrive in this point self.stored_data must be reseted because
        # will be a new city information
        self.stored_data_count = 0

    def getStreamData(self, query=None, save_path=None):
        """Method to get data in almost real-time.

        The tweets that match with query will be saved as json data in a *.tsv file
        
        Args:
            query (str) : Keywords or expression that a tweet must has.
            save_path (str) : path to be saved the file generated with json data

        """
        if(save_path is None):
            print(colored('Error: You must define save_path', 'red'))
            sys.exit()

        self.listenerHanlder = StreamListenerHandler(save_path)
        self.streamData = tweepy.Stream(self.auth, self.listenerHanlder)
        self.streamData.filter(track=query)


# Method use only Stream Mode
class StreamListenerHandler(tweepy.StreamListener):
    """Listener Method to be listen Twitter's server.
    
    When the Twitter's server get a tweet that match with a query passed in getStreamData method
    will be send for our app, the data is sent with json data format.

    """

    def __init__(self, save_path):
        """Method to set log file path."""
        self.log_file = open(save_path + 'log_.tsv', 'a', encoding='utf8')

    def on_data(self, data):
        """Method called each data sent from TWitter's server ."""
        jsonencoded = json.loads(data)

        print(jsonencoded["created_at"], str(jsonencoded["user"][
              "id"]), jsonencoded["id_str"], jsonencoded["user"]["name"])

        self.log_file.write((jsonencoded["created_at"]) + '\t' + str(jsonencoded["user"]["id"]) + '\t' +
                            (jsonencoded["user"]["lang"]) + '\t' + (jsonencoded["text"]) + '\t' +
                            str(jsonencoded["id_str"]) + '\t' + str(jsonencoded["user"]["screen_name"]) + '\t' + str(jsonencoded["user"]["name"]) + '\n')
        return True

    def on_error(self, status):
        """Method to print the error message ."""
        print(colored(status), 'red')

    # def removeNone(self, var):
    #     try:
    #         var = var + "1"
    #     except (AttributeError, ValueError, TypeError):
    #         print(colored('NoneType detected', 'red'))



from SocialCrawler import NewExtractorData

client_id = "GEQQO0MJPKLUTBIZHERRYQJUCTU5W2MGFAHZKCDB3LYB1GZI"
client_secret = "QMXESMEH1VKEEKUQ0EBXNCJ4IWCK0EYHA0MUHNVBMGSWXRUZ"

file_name = "Brasillog_from_Cuiaba__2016-08-28__2016-09-05.tsv"



test = NewExtractorData.NewExtractor(client_id,client_secret,developer_email="josiel.wirlino@gmail.com",developer_password ="j0sielengenheiro")

indice = 0
# while( indice < 4):

test.settings(mode="v1",
			  out_file_name="cuiaba_final_agosto",
			  out_path_file="/home/wirlino/Documents/Git/SocialCrawler/SocialCrawler/",
			  input_file="/home/wirlino/Documents/Git/SocialCrawler/SocialCrawler/"+file_name			  
			  )

""" We are setting mode to v1 that means the file was created by or Collector or CollectorV2
    output file created will be log_new_york.tsv and will be saved in ./Log/
    the input file ( that contains t.co url ) are localed in ./Log/Brasil/ folder
"""

test.consult_foursquare()



# -*- coding: UTF-8 -*-
"""Extractor class to retrieve data from Foursquare.

# (c) J.Wirlino and A.Adativa
# Date Sept 1th 2016
# https://github.com/JosielWirlino/SocialCrawler

"""

from termcolor import colored
# import json
import sys
import urllib.request
from urllib.error import URLError
from pip._vendor.requests.exceptions import HTTPError
import time
import requests

class Extractor:
    """Read log file from Collector .

    ( or files that contains a swarm url code)

    This class use tools and keywords from Foursquare API Endpoints.

        Note:
            If has a another file that was not created by Collector you must
            have set.column parameter with a integer saying the file column that
            has swarm url. All file input must be *.tsv .

        Args:
            see def __init__ docstring
    """
   
    def __init__(self,foursquare_client_id=None, foursquare_client_secret=None,foursquare_version="20140806%20", foursquare_mode="swarm"):
        """Class constructor.
        
        Method to init all minimum settings necessary to get data froum Foursquare.
        
        Note:
            
            foursquare_mode and foursquare_version parameter are two specials parameters 
            to control over the kind of that Foursquare returns.
            see https://developer.foursquare.com/overview/versioning

            To see how can get crendetials see https://developer.foursquare.com/overview/auth

        Args:
            foursquare_client_id (str) : Foursquare's app client id
            foursquare_client_secret (str) : Foursquare's app client secret
            foursquare_version (str) : kind of Foursquare returns, by default "20140806%20" .
            foursquare_mode (str) : foursquare mode answer. Valid value: "swarm"  or "foursquare" by default "swarm"

        Attributes:
            foursquare_client_id (str) : Foursquare's app client id
            foursquare_client_secret (str) : Foursquare's app client secret
            foursquare_version (str) : kind of Foursquare returns
            foursquare_mode (str) : foursquare mode answer
                                    valid value: "swarm"  or "foursquare"
        """
        foursquare_mode_valid_value = {'swarm','foursquare'}

        if( foursquare_client_id is None or foursquare_client_secret is None ):
            print(colored('Any parameter can be None.','red'))
            sys.exit()

        if( foursquare_mode not in foursquare_mode_valid_value ):
            print(colored('fourquare_mode invalid value.','red'))
            sys.exit()

        self._foursquare_client_id = foursquare_client_id
        self._foursquare_client_secret = foursquare_client_secret
        self._foursquare_version = foursquare_version
        self._foursquare_mode = foursquare_mode

    # @property
    # def file_name(self):
    #     """File name that will be used to save the file created when run."""
    #     return self._file_name
    
    # @property
    # def mode(self):
    # 	"""Mode that class was setted."""
    # 	return self._mode
    
    # @property
    # def path_file(self):
    # 	"""folder where the file will be saved."""
    # 	return self._path_file
    
    def settings(self,mode=None,out_file_name=None,out_path_file=None,column=4,input_file=None):
        """Class constructor.

        Method to read log file created by Collector class
        and get Swarm's url code and consult Foursquare to retrieve
        information about avenue, local type, shared_count and all information that is
        returned than will save in a file called log_<out_file_name>.tsv

        Args:
            out_file_name (str) : File name to be created.
            out_path_file (str) : File path where the file must be saved.
            mode (str) : json file version.
                         mode valid value:
                         - v1 if the file was created from Collector class or CollectorV2 class
                         - v2 if was not created by any Collector but has the Swarm url
                                  if the value is v2 the "column" parameter must be setted
            column (int) : column that contain the Swarm url. This field is only
                           verified if "mode" is v2 .
            input_file(str) : file that contains swarm url code

        """
        mode_valid_value = {'v1','v2'}

        if(mode is None or out_file_name is None or out_path_file is None or input_file is None):
            print(colored('Any parameter can be None except column field iff mode is not v2. ','red'))
            sys.exit()

        if(mode not in mode_valid_value ):
            print(colored('mode value invalid.','red'))
            sys.exit()

        self._file_name = out_file_name
        self._path_file = out_path_file
        self._mode = mode
        self._input_file = input_file
        self._column = column
        if(mode == "v2"):
            if(column is None ):
                print(colored("When mode is v2, column field must be assigned",'red'))
                sys.exit()
            self._column = column

    def consultFoursquare(self):
        """Method to access foursquare and use resolveId.

        This method is called to each line read from input file, so
        all attributes of credentials function must be setted correctly

        resolvedId is a function from Foursquare API Endpoints to get data from a swarm url code.
        see https://developer.foursquare.com/docs/
        """
        swarm_prefix = "https://www.swarmapp.com/c/"
        url_venue = 'https://api.foursquare.com/v2/venues/'
        url_resolveID = 'https://api.foursquare.com/v2/checkins/resolve?shortId='

        if(self._mode is None or self._file_name is None or self._path_file is None or self._input_file is None):
            print(colored('One or more attributes are None. Firstly you must call settings method and than you call this method ','red'))
            sys.exit()

        __out = open( self._path_file + '/' + self._file_name + '.tsv','a')
        
        #set header
        __out.write( 'checkin_id' \
                    + '\t' + 'checkin_createdAt' \
                    + '\t' + 'user_id' \
                    + '\t' + 'user_gender' \
                    + '\t' + 'swarm_key' \
                    + '\t' + 'venue_id' \
                    + '\t' + 'venue_coord' \
                    + '\t' + 'venue_categories_id' \
                    + '\t' + 'venue_categories_name' \
                    + '\t' + 'venue_stats_checkinsCount' \
                    + '\t' + 'venue_stats_userCount' \
                    +'\n')

        for line in open( self._input_file,'r'): #read each line from input file
            
            line_split = line.split('\t')
            
            # print(line + "URL T.CO " + line_split[self._column])
            try:
            
                t_co_url = line_split[self._column] #get tweet text
                
                idx = t_co_url.index('https://t.co/')

                t_co_url = t_co_url[ idx : idx+23 ] #get all t.co url

                print("Trying resolve "+t_co_url)

                swarm_resolved_url =  urllib.request.urlopen( t_co_url ) # try resolve swarm short url 
            
            except HTTPError as e:
                print(colored('URLLIB URLOPEN FAILED ERROR CODE %s' %e.code,'red'))
                continue
            
            except ValueError as e:
                print(colored(' SWARM SHORT URL ERROR ','red'))
                print(e)
                continue
            except URLError as e:
                print(colored(' URL ERROR ','red'))
                print(e)
                continue
            except ConnectionResetError as e:
                print(colored(' CONNECTION RESET ERROR ','red'))
                print(e)
                continue
            except ConnectionError as e:
                print(colored(' CONNECTION ERROR ','red'))
                print(e)
                continue
            except ConnectionAbortedError as e:
                print(colored(' CONNECTION ABORTED ERROR ','red'))
                print(e)
                continue
            except ConnectionRefusedError as e:
                print(colored(' CONNECTION REFUSED ERROR ','red'))
                print(e)
                continue
            except :
                print(colored(' URLLIB ERROR','red'))
                continue

            if swarm_prefix not in swarm_resolved_url.url: #if do not have swarm key code
                print(swarm_resolved_url.url)
                print(colored('SWARM URL ERROR','red') )
                continue

            print("resolved sucessfully. Link resolved is "+swarm_resolved_url.url )

            key =  swarm_resolved_url.url[ len(swarm_resolved_url.url) - 11 : ] #get string after www.swarmapp.com/
            
            try:

                response = requests.get( url_resolveID + key + 
                                      '&client_id=' + self._foursquare_client_id +
                                      '&client_secret=' + self._foursquare_client_secret +
                                      '&v=' + self._foursquare_version +
                                      '&m=' + self._foursquare_mode )

                swarm_4square_data = response.json()
                

            except HTTPError as e:
                print(colored('HTTPERROR ','red'))
                print(colored(e,'red'))
                continue
            except :
                print(colored('FOURSQUARE REQUESTS GET ERROR ','red'))
                # hour = time.strftime("%H")
                # minute = time.strftime("%M")
                # second = time.strftime("%S")
                # print(colored('STOP REQUEST IN ' + str(hour+1) + ':' + str(minute+1) + ':' + str(second) , 'red' ) )
                print(colored('Wait 15 minutes to request again ','red'))
                time.sleep(960) #sleep 1 hour and 1 minute

                continue

            try:
                v_id = swarm_4square_data['response']['checkin']['venue']['id']
                venue_request = requests.get( url_venue + v_id +
                                                  '?client_id=' + self._foursquare_client_id +
                                                  '&client_secret=' + self._foursquare_client_secret +
                                                  '&v=' + self._foursquare_version)

                venue_data = venue_request.json()
            except HTTPError as e:
                print(colored('HTTPERROR ','red'))
                print(colored(e,'red'))
                continue

            except :
                print(colored('VENUE REQUESTS GET ERROR ','red'))
                # print(colored('STOP REQUEST IN ' + str(time.strftime("%H:%M:%S"),'red') ) )
                
                # hour = time.strftime("%H")
                # minute = time.strftime("%M")
                # second = time.strftime("%S")
                print(colored('Wait one 15 minutes to request again ','red'))
                # print('Next request '+ str(hour+1) + ':' + str(minute+1) + ':' + str(second))
                # print(colored('STOP REQUEST IN ' + str(hour+1) + ':' + str(minute+1) + ':' + str(second) , 'red' ) )
                time.sleep(960) #sleep 15 minutes
                continue    
                # if( swarm_4square_data['meta']['code'] == "200"): #data yet available in foursquare dataset

            try:
                checkin_id = swarm_4square_data['response']['checkin']['id']
            except:
                print(colored('SWARM CHECKIN ERROR [PARSER CHECKIN_ID]','red'))
                continue
            
            try:    
                checkin_createdAt = swarm_4square_data['response']['checkin']['createdAt']
            except:
                print(colored('SWARM CHECKIN ERROR [PARSER CREATEDAT]','red'))
                continue

            try:
                user_id = swarm_4square_data['response']['checkin']['user']['id']
            except:
                print(colored('SWARM CHECKIN ERROR [PARSER USER_ID]','red'))
                continue

            try:
                user_gender = swarm_4square_data['response']['checkin']['user']['gender']
            except:
                print(colored('SWARM CHECKIN ERROR [PARSER USER_GENDER]','red'))
                continue

            print("Foursquare and Venue requests success.")
            print("Saving Foursquare data.")
            __out.write( str(checkin_id) + \
                        '\t' + str(checkin_createdAt) \
                        + '\t' + str(user_id) + '\t' \
                        + user_gender \
                        + '\t' + key + '\t' )
            # write data about user

            try:
                venue_id = venue_data['response']['venue']['id']
            except:
                print(colored('VENUE ERROR [PARSER VENUE_ID]','red'))
                continue

            try:
                venue_lat = venue_data['response']['venue']['location']['lat']
            except:
                print(colored('VENUE ERROR [PARSER VENUE_LOCATION_LAT]','red'))
                __out.write('\n')
                continue

            try:
                venue_lng = venue_data['response']['venue']['location']['lng']
            except:
                print(colored('VENUE ERROR [PARSER VENUE_LOCATION_LNG]','red'))
                __out.write('\n')
                continue

            try:
                categorie_id = venue_data['response']['venue']['categories'][0]['id']
            except:
                print(colored('VENUE ERROR [PARSER VENUE_CATEGORIE_ID]','red'))
                __out.write('\n')
                continue

            try:
                categorie_name = venue_data['response']['venue']['categories'][0]['name']
            except:
                print(colored('VENUE ERROR [PARSER VENUE_CATEGORIE_NAME]','red'))
                __out.write('\n')
                continue

            try:
                stats_checkins_count = venue_data['response']['venue']['stats']['checkinsCount']
            except:
                print(colored('VENUE ERROR [PARSER VENUE_STATS_CHECKINSCOUNT]','red'))
                __out.write('\n')
                continue

            try:
                stats_user_count = venue_data['response']['venue']['stats']['usersCount']
            except:
                print(colored('VENUE ERROR [PARSER VENUE_STATS_USERSCOUNT]','red'))
                __out.write('\n')
                continue
            
            print("Saving Venue data.")
            __out.write( str(venue_id) \
                        + '\t' + str(venue_lat) + ',' + str(venue_lng) \
                        + '\t' + str(categorie_id) + '\t' \
                        + categorie_name + '\t' \
                        + str(stats_checkins_count) \
                        + '\t' + str(stats_user_count) \
                        + '\n')
            # break
        __out.close()















from HackFoursquare import Hacking
import json
hack = Hacking("josiel.wirlino@gmail.com","j0sielengenheiro")
hack.open_browser()

venue = hack.get_venue_detail("5611c005498eacfce78eceee")
venue2 = json.loads(venue)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# (c) J.Wirlino and A.Adativa
# Date July 15th 2016
# https://github.com/JosielWirlino/SocialCrawler

from setuptools import setup

PACKAGE = "SocialCrawler"
NAME = "SocialCrawler"
DESCRIPTION = "This package allow you get data from Twitter, Swarm(check-ins) and Foursquare"
AUTHOR = "J.Wirlino and A. Adativa"
AUTHOR_EMAIL = "josiel.wirlino@gmail.com, alice.adativa@gmail.com"
URL = "http://github.com/JosielWirlino/SocialCrawler"
VERSION = __import__(PACKAGE).__version__


with open('README.rst') as file:
	long_description = file.read()

setup( name=NAME,
	version=VERSION,
	description=DESCRIPTION,
	long_description=long_description,
	url=URL,
	author=AUTHOR,
	author_email=AUTHOR_EMAIL,
	license='GNU',
	packages=[PACKAGE],
	install_requires=['tweepy',
					  'pyvirtualdisplay',
					  'termcolor',
					  'selenium',
					  'requests>=2.4.3',
					  ],
	classifiers=[
				'Intended Audience :: Developers',
				'Intended Audience :: Science/Research',
				'License :: OSI Approved :: GNU General Public License (GPL)',
				'Programming Language :: Python :: 3',
				'Programming Language :: Python :: 3.4',
				'Programming Language :: Python :: 3.5',
				'Topic :: Scientific/Engineering :: Information Analysis'
				],
	zip_safe=False
	)